/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

/**
 * Main class of the application.
 */
public class Cable.App : Granite.Application {

    static bool option_reset = false;
    static bool option_version = false;

    static const OptionEntry[] option_entries = {
        { "reset", 0, 0, OptionArg.NONE, ref option_reset,
          N_("Reset all settings and configuration to default"), null },

        { "version", 0, 0, OptionArg.NONE, ref option_version,
          N_("Print the version number"), null },

        { null }
    };

    static Gee.ArrayList <Controller.Window> controllers {
        get; set;
        default = new Gee.ArrayList <Controller.Window> ();
    }

    public static const string TITLE = "Cable";

    construct {
        build_data_dir = Constants.DATADIR;
        build_pkg_data_dir = Constants.PKGDATADIR;
        build_release_name = Constants.RELEASE_NAME;
        build_version = Constants.VERSION;
        build_version_info = Constants.VERSION_INFO;

        program_name = "Cable";
        exec_name = "cable";

        app_years = "2012-2013";
        app_icon = "internet-chat";
        app_launcher = "cable.desktop";
        application_id = "net.launchpad.cable";

        main_url = "https://code.launchpad.net/cable";
        bug_url = "https://bugs.launchpad.net/cable";
        help_url = "https://answers.launchpad.net/cable";
        translate_url = "https://translations.launchpad.net/cable";

        about_authors = {"Akshay Shekher <voldyman666@gmail.com>",
                         "Auroral Xylon <avlabs314@gmail.com>",
                         "Eduard Gotwig <gotwig@ubuntu.com>",
                         "Julien Spautz <spautz.julien@gmail.com>",
                         "Tom Beckmann <tomjonabc@gmail.com>"};
        about_documenters = {};
        about_artists = {"Harvey Cabaguio <harveycabaguio@gmail.com"};
        about_comments = _("Development release, not all features implemented");
        about_translators = "Launchpad Translators";
        about_license_type = Gtk.License.GPL_3_0;
    }

    public App () {
        Services.Logger.init ();
        Services.Notify.init ();
        Services.Launcher.init ();
        Settings.init ();
        Utils.Network.init ();
        Utils.Identity.init ();
        Irc.init ();
    }

    public override void activate () {
        var window = new Widgets.Window (this);
        var controller = new Controller.Window (window);

        add_window (window);
        controllers.add (controller);
    }

    public static int main (string[] args) {
        var context = new GLib.OptionContext ("");
        context.set_help_enabled (true);
        context.add_main_entries (option_entries, Constants.GETTEXT_PACKAGE);

        try {
            context.parse (ref args);
        } catch (GLib.OptionError e) {
            warning (e.message);
            warning ("Run '%s --help' to see a full list of available command line options.\n", args[0]);
            return 0;
        }

        if (option_version) {
            // XXX
        }

        if (option_reset) {
            // XXX
        }

        var cable = new Cable.App ();
        return cable.run (args);
    }

    /*internal void translations () {
        var description = N_("Stylish IRC Client");
        var generic = N_("IRC Client");
    }*/
}


