/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Controller.PrivateChat : GLib.Object {
    
    public weak Server server;
    public View.PrivateChat view;
    string id;
    
    public PrivateChat (Server server, string id, string message = "") {
        this.id = id;
        this.server = server;
        
        server.view.add_private_chat (id, id);
        view = server.view.get_chat (id) as View.PrivateChat;
        view.nick = server.nick;
        
        if (message != "")
            on_backend_message (id, server.nick, message);
        
        connect_signals ();
    }
    
    void connect_signals () {
        server.backend.event_message.connect (on_backend_message);
        view.send_message.connect (on_frontend_message);
        server.window.view.selected.connect (on_chat_selected);
    }
    
    void on_backend_message (string from, string target, string message) {
        if (is_not_me (target))
            return;
            
        if (is_not_correspondent (from))
            return;
        
        var nick_from = from.split ("!")[0];
        
        view.message (nick_from, message);
        
        if (view.has_focus == false) {
            Services.Notify.send (nick_from, message);
            Services.Launcher.jobs++;
            Services.Launcher.urgent = true;
        }
        
        if (server.window.view.current_chat != view)
            view.unread_messages++;
    }
    
    void on_frontend_message (string message) {
        server.backend.send_message (id.split("!")[0], message);
        view.message (server.nick, message);
        view.entry = "";
    }
    
    void on_chat_selected (View.Chat chat) {
        if (chat == view)
            view.unread_messages = 0;
    }
    
    bool is_not_me (string nick) {
        return nick != server.nick;
    }
    
    bool is_not_correspondent (string from) {
        return from.split ("!")[0] != id;
    }
}
