/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Controller.Server : GLib.Object {

    public weak Window window;
    public View.Server view;
    public Irc.Server backend;
    
    public string nick; // FIXME not reliable
    public string address; // FIXME not reliable
    public string name;

    public Gee.HashMap <string, Channel> channels {
        get; set; default = new Gee.HashMap <string, Channel>  ();
    }
    
    public Gee.HashMap <string, PrivateChat> private_chats {
        get; set; default = new Gee.HashMap <string, PrivateChat>  ();
    }

    public Server (string address, string nick, Window window) {
        this.backend = Irc.Server.get_or_create (address, nick);

        this.name = backend.name;
        this.window = window;
        this.nick = nick;
        this.address = address;
        
        if (!backend.connected)
            backend.send_connect ();

        this.view = window.view.add_server (@"$(backend.network.name) - $(nick)", backend.name);

        connect_frontend_signals ();
        connect_backend_signals ();
    }
    
    void connect_frontend_signals () {
        view.toggle_away.connect (() =>  {
            if (backend.away) {
                backend.send_back ();
            } else {
                backend.send_away ();
            }
        });

        view.part_all.connect (() => {
            foreach (var channel in channels.values)
                channel.backend.send_part ();
        });
    }
    
    void connect_backend_signals () {
        backend.event_user_away.connect ((from, away) => {
            if (backend.is_me (from)) {
                view.away = away;
            }
            
            var nick = from.split("!")[0];
            
            foreach (var channel in channels.values)
                if ( nick in channel.backend.nicks )
                    channel.view.set_away_status (nick, away);
        });
        
        backend.event_quit.connect ((from, message) => {
            var nick = from.split ("!")[0];
            
            foreach (var channel in channels.values)
                channel.view.remove_user (nick, message);
        });
        
        backend.event_message.connect ((from, target, message) => {
            if (target != nick)
                return;
            
            var from_nick = from.split ("!")[0];
            
            open_new_chat_with (from_nick, message);
        });
    }
    
    public void open_new_chat_with (string from_nick, string message = "") {
        if (from_nick == backend.nick)
            return;
            
        if (!private_chats.has_key (from_nick)) {
            var chat = new PrivateChat (this, from_nick, message);
            private_chats [from_nick] = chat;
        }
    }
}
