/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

/**
 * Add or edit an identity.
 */
public class Cable.Dialogs.ManageIdentity : Granite.Widgets.LightWindow {

    /**
     * @param identity Resulting identity.
     */
    public signal void done (Utils.Identity identity);

    /**
     * @param identity Identity to be edited or null if you want to create
     * a new one.
     */
    public ManageIdentity (Utils.Identity? identity = null) {
    
        var new_identity = identity == null;
        var in_use = (new_identity) ? false : identity.servers.size > 0;
        
        this.modal = true;
        this.window_position = Gtk.WindowPosition.CENTER;
        
        var grid = new Gtk.Grid ();
        grid.margin = 12;
        grid.column_spacing = 6;
        grid.row_spacing = 6;

        var nick_name = new Gtk.Entry ();
        nick_name.placeholder_text = _("johndoe (Without spaces/special characters)");
        nick_name.width_request = 250;
        nick_name.sensitive = !in_use;
        
        var real_name = new Gtk.Entry ();
        real_name.placeholder_text = _("John Doe");
        real_name.sensitive = !in_use;
        
        var part_message = new Gtk.Entry ();
        part_message.placeholder_text = _("Good night everyone!");
        
        var away_message = new Gtk.Entry ();
        away_message.placeholder_text = _("Try again later!");
        
        var quit_message = new Gtk.Entry ();
        quit_message.placeholder_text = _("Good night everyone!");

        grid.attach (new Utils.Label.right (_("Nick Name:"), !in_use), 0, 0, 1, 1);
        grid.attach (nick_name, 1, 0, 1, 1);
        grid.attach (new Utils.Label.right (_("Real Name:"), !in_use), 0, 1, 1, 1);
        grid.attach (real_name, 1, 1, 1, 1);
        grid.attach (new Utils.Label.right (_("Part Message:")), 0, 2, 1, 1);
        grid.attach (part_message, 1, 2, 1, 1);
        grid.attach (new Utils.Label.right (_("Away Message:")), 0, 3, 1, 1);
        grid.attach (away_message, 1, 3, 1, 1);
        grid.attach (new Utils.Label.right (_("Quit Message:")), 0, 4, 1, 1);
        grid.attach (quit_message, 1, 4, 1, 1);

        var cancel_button = new Gtk.Button.from_stock (Gtk.Stock.CANCEL);
        Gtk.Button apply_button;
        
        if (new_identity) {
            apply_button = new Gtk.Button.from_stock (Gtk.Stock.ADD);
            apply_button.get_style_context ().add_class ("affirmative");
        } else {
            apply_button = new Gtk.Button.from_stock (Gtk.Stock.APPLY);
        }
        
        apply_button.sensitive = !new_identity;
        
        var button_box = new Gtk.ButtonBox (Gtk.Orientation.HORIZONTAL);
        button_box.layout_style = Gtk.ButtonBoxStyle.END;
        button_box.add (cancel_button);
        button_box.add (apply_button);

        grid.attach (button_box, 0, 5, 2, 1);

        this.add (grid);
        this.title = (new_identity) ? _("Add Identity") : _("Edit Identity");

        if (!new_identity) {
            nick_name.text = identity.nick_name;
            real_name.text = identity.real_name;
            part_message.text = identity.part_message;
            away_message.text = identity.away_message;
            quit_message.text = identity.quit_message;
        }

        nick_name.changed.connect (() => {
            apply_button.sensitive = (nick_name.text != "");
        });

        nick_name.activate.connect (() => apply_button.clicked);
        real_name.activate.connect (() => apply_button.clicked);
        part_message.activate.connect (() => apply_button.clicked);
        away_message.activate.connect (() => apply_button.clicked);
        quit_message.activate.connect (() => apply_button.clicked);
        
        apply_button.clicked.connect (() => {
            if (new_identity) {
                done (create_identity (nick_name.text, real_name.text, part_message.text));
            } else {
                identity.part_message = part_message.text.strip ();
                identity.away_message = away_message.text.strip ();
                identity.quit_message = quit_message.text.strip ();
                done (identity);
            }
            
            this.destroy ();
        });
        
        cancel_button.clicked.connect (() => {
            this.destroy ();
        });
    }

    static Utils.Identity create_identity (string nick_name, string real_name, string part_message)
        requires (nick_name.strip () != "") {

        if (nick_name.strip () == "")
            return (Utils.Identity) null;

        if (real_name.strip () == "")
            real_name = null;
        if (part_message.strip () == "")
            part_message = null;

        return new Utils.Identity (nick_name, real_name, part_message);
    }
}
