/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

/**
 * Add or edit a network.
 */
public class Cable.Dialogs.ManageNetwork : Granite.Widgets.LightWindow {

    /**
     * @param network Resulting network.
     */
    public signal void done (Utils.Network network);

    /**
     * @param network Network to be edited or null if you want to create
     * a new one.
     */
    public ManageNetwork (Utils.Network? network = null) {
        var new_network = network == null;
        var in_use = (new_network) ? false : network.servers.size > 0;
    
        this.modal = true;
        this.window_position = Gtk.WindowPosition.CENTER;
        
        var grid = new Gtk.Grid ();
        grid.margin = 12;
        grid.column_spacing = 6;
        grid.row_spacing = 6;

        var address = new Gtk.Entry ();
        address.placeholder_text = _("irc.network.net");
        address.width_request = 250;
        address.sensitive = !in_use;
        
        var name = new Gtk.Entry ();
        name.placeholder_text = _("Network IRC");
        
        var password = new Gtk.Entry ();
        password.placeholder_text = _("Optional");
        password.caps_lock_warning = true;
        password.set_visibility (false);

        grid.attach (new Utils.Label.right (_("Address:"), !in_use), 0, 0, 1, 1);
        grid.attach (address, 1, 0, 1, 1);
        grid.attach (new Utils.Label.right (_("Name:")), 0, 1, 1, 1);
        grid.attach (name, 1, 1, 1, 1);
        grid.attach (new Utils.Label.right (_("Password:")), 0, 2, 1, 1);
        grid.attach (password, 1, 2, 1, 1);

        var cancel_button = new Gtk.Button.from_stock (Gtk.Stock.CANCEL);
        Gtk.Button apply_button;
        
        if (network == null) {
            apply_button = new Gtk.Button.from_stock (Gtk.Stock.ADD);
            apply_button.get_style_context ().add_class ("suggested-action");
        } else {
            apply_button = new Gtk.Button.from_stock (Gtk.Stock.APPLY);
        }
        
        apply_button.sensitive = (network != null);
        
        var button_box = new Gtk.ButtonBox (Gtk.Orientation.HORIZONTAL);
        button_box.layout_style = Gtk.ButtonBoxStyle.END;
        button_box.add (cancel_button);
        button_box.add (apply_button);

        grid.attach (button_box, 0, 3, 2, 1);

        this.add (grid);
        this.title = (network == null) ? _("Add Network") : _("Edit Network");

        if (network != null) {
            address.text = network.address;
            name.text = network.name;
            password.text = network.password;
        }

        address.changed.connect (() => {
            apply_button.sensitive = (address.text != "");
        });

        apply_button.clicked.connect (() => {
            if (network == null) {
                done (create_network (address.text, name.text, password.text));
            } else {
                network.name = name.text.strip ();
                network.password = password.text.strip ();
                done (network);
            }
            
            this.destroy ();
        });
        
        cancel_button.clicked.connect (() => {
            this.destroy ();
        });
    }

    Utils.Network create_network (string address, string name, string password)
        requires (address.strip () != "") {

        if (address.strip () == "")
            return (Utils.Network) null;

        if (name.strip () == "")
            name = null;
        if (password.strip () == "")
            password = null;

        return new Utils.Network (address, name, password);
    }
}
