/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Dialogs.Preferences : Granite.Widgets.LightWindow {

    public Preferences () {
        this.title = _("Preferences");
        this.window_position = Gtk.WindowPosition.CENTER;

        var static_notebook = new Granite.Widgets.StaticNotebook (false);
        static_notebook.margin = 12;
        static_notebook.margin_top = 6;
        static_notebook.expand = true;
        static_notebook.append_page (new NetworkPrefs (), new Gtk.Label (_("Networks")));
        static_notebook.append_page (new IdentityPrefs (), new Gtk.Label (_("Identities")));

        this.add (static_notebook);
    }
}

public class Cable.Dialogs.NetworkPrefs : Gtk.Grid {

    public NetworkPrefs () {
        var tree = new Gtk.TreeView ();
        var scroll = new Gtk.ScrolledWindow (null, null);

        sync (tree);

        var cell = new Gtk.CellRendererText ();
        tree.insert_column_with_attributes (-1, _("Name"), cell, "text", 0);
        tree.insert_column_with_attributes (-1, _("Address"), cell, "text", 1);
        tree.insert_column_with_attributes (-1, _("Used"), cell, "text", 2);

        scroll.set_size_request (300, 300);
        scroll.hscrollbar_policy = Gtk.PolicyType.AUTOMATIC;
        scroll.vscrollbar_policy = Gtk.PolicyType.AUTOMATIC;
        scroll.shadow_type = Gtk.ShadowType.IN;
        scroll.expand = true;
        scroll.add (tree);

        var toolbar = new Gtk.Toolbar();
        toolbar.set_style(Gtk.ToolbarStyle.ICONS);
        toolbar.set_icon_size(Gtk.IconSize.SMALL_TOOLBAR);
        toolbar.set_show_arrow(false);
        toolbar.hexpand = true;

        scroll.get_style_context().set_junction_sides(Gtk.JunctionSides.BOTTOM);
        toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR);
        toolbar.get_style_context().set_junction_sides(Gtk.JunctionSides.TOP);

        var add_button = new Gtk.ToolButton (null, _("Add…"));
        var remove_button = new Gtk.ToolButton (null, _("Remove"));
        var edit_button = new Gtk.ToolButton (null, _("Edit…"));

        add_button.set_tooltip_text (_("Add…"));
        remove_button.set_tooltip_text (_("Remove"));
        edit_button.set_tooltip_text (_("Edit…"));
        
        add_button.set_icon_name ("list-add-symbolic");
        remove_button.set_icon_name ("list-remove-symbolic");
        edit_button.set_icon_name ("document-properties-symbolic");
        
        toolbar.insert (add_button, -1);
        toolbar.insert (remove_button, -1);
        //TODO toolbar.insert (edit_button, -1);

        this.attach (scroll, 0, 0, 1, 1);
        this.attach (toolbar, 0, 1, 1, 1);

        add_button.clicked.connect (() => {
            var dialog = new Dialogs.ManageNetwork ();
            dialog.done.connect ((network) => this.sync (tree));
            dialog.show_all ();
        });

        remove_button.clicked.connect (() => {
            Gtk.TreePath? path;
            tree.get_cursor (out path, null);

            if (path == null)
                return;

            var index = path.get_indices ()[0];
            Utils.Network.remove (Utils.Network.all[index].address);
            sync (tree);
        });
        
        tree.row_activated.connect ((path) => {
            var index = path.get_indices ()[0];
            var dialog = new Dialogs.ManageNetwork (Utils.Network.all[index]);
            dialog.done.connect (() => this.sync (tree));
            dialog.show_all ();
        });
    }

    private void sync (Gtk.TreeView tree) {
        var store = new Gtk.ListStore (3, typeof (string), typeof (string), typeof (string));

        Gtk.TreeIter iter;

        foreach (var network in Utils.Network.all) {
            var in_use = network.servers.size > 0 ? "Used" : "";
            store.append (out iter);
            store.set (iter, 0, network.name, 1, network.address, 2, in_use, -1);
        }

        tree.model = store;
    }
}

 public class Cable.Dialogs.IdentityPrefs : Gtk.Grid {

    public IdentityPrefs () {
        var tree = new Gtk.TreeView ();
        var scroll = new Gtk.ScrolledWindow(null, null);

        sync (tree);

        var cell = new Gtk.CellRendererText ();
        tree.insert_column_with_attributes (-1, _("Nick"), cell, "text", 0);
        tree.insert_column_with_attributes (-1, _("Real Name"), cell, "text", 1);

        scroll.set_size_request (300, 300);
        scroll.hscrollbar_policy = Gtk.PolicyType.AUTOMATIC;
        scroll.vscrollbar_policy = Gtk.PolicyType.AUTOMATIC;
        scroll.shadow_type = Gtk.ShadowType.IN;
        scroll.expand = true;
        scroll.add (tree);

        var toolbar = new Gtk.Toolbar();
        toolbar.set_style(Gtk.ToolbarStyle.ICONS);
        toolbar.set_icon_size(Gtk.IconSize.SMALL_TOOLBAR);
        toolbar.set_show_arrow(false);
        toolbar.hexpand = true;

        scroll.get_style_context().set_junction_sides(Gtk.JunctionSides.BOTTOM);
        toolbar.get_style_context().add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR);
        toolbar.get_style_context().set_junction_sides(Gtk.JunctionSides.TOP);

        var add_button    = new Gtk.ToolButton (null, _("Add"));
        var remove_button = new Gtk.ToolButton (null, _("Remove"));

        add_button.set_tooltip_text (_("Add"));
        remove_button.set_tooltip_text (_("Remove"));

        add_button.set_icon_name ("list-add-symbolic");
        remove_button.set_icon_name ("list-remove-symbolic");

        toolbar.insert (add_button, -1);
        toolbar.insert (remove_button, -1);

        this.attach (scroll, 0, 0, 1, 1);
        this.attach (toolbar, 0, 1, 1, 1);

        add_button.clicked.connect (() => {
            var dialog = new Dialogs.ManageIdentity ();
            dialog.done.connect (() => this.sync (tree));
            dialog.show_all ();
        });

        remove_button.clicked.connect (() => {
            Gtk.TreePath? path;
            tree.get_cursor (out path, null);

            if (path == null)
                return;

            var index = path.get_indices ()[0];
            Utils.Identity.remove (Utils.Identity.all[index].nick_name);
            sync (tree);
        });

        tree.row_activated.connect ((path) => {
            var index = path.get_indices ()[0];
            var dialog = new Dialogs.ManageIdentity (Utils.Identity.all[index]);
            dialog.done.connect (() => this.sync (tree));
            dialog.show_all ();
        });
    }

    private void sync (Gtk.TreeView tree) {
        var store = new Gtk.ListStore (2, typeof (string), typeof (string));

        Gtk.TreeIter iter;

        foreach (var identity in Utils.Identity.all) {
            store.append (out iter);
            store.set (iter, 0, identity.nick_name, 1, identity.real_name, -1);
        }

        tree.model = store;
    }
}
