/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

/**
 * This is a dbus interface to the maki daemon from the SushiIRC project.
 * Use the methods to send requests or messages to the server and the
 * signals to react to the server's replies.
 */
[DBus (name = "de.ikkoku.sushi", timeout = 120000)]
public interface Cable.Irc.Maki : GLib.Object {

    [DBus (name = "action")]
    public abstract void action (string server, string channel, string message) throws GLib.IOError;

    [DBus (name = "away")]
    public abstract void away (string server, string message) throws GLib.IOError;

    [DBus (name = "back")]
    public abstract void back (string server) throws GLib.IOError;

    [DBus (name = "channel_nicks")]
    public abstract void channel_nicks (string server, string channel, out string[] nicks, out string[] prefixes) throws GLib.IOError;

    [DBus (name = "channel_topic")]
    public abstract string channel_topic (string server, string channel) throws GLib.IOError;

    [DBus (name = "channels")]
    public abstract string[] channels (string server) throws GLib.IOError;

    [DBus (name = "config_get")]
    public abstract string config_get (string group, string key) throws GLib.IOError;

    [DBus (name = "config_set")]
    public abstract void config_set (string group, string key, string value) throws GLib.IOError;

    // connect to a server known to maki
    [DBus (name = "connect")]
    public abstract void connect (string server) throws GLib.IOError;

    [DBus (name = "ctcp")]
    public abstract void ctcp (string server, string target, string message) throws GLib.IOError;

    [DBus (name = "dcc_send")]
    public abstract void dcc_send(string server, string target, string path) throws GLib.IOError;

    [DBus (name = "dcc_sends")]
    public abstract void dcc_sends (out uint64[] ids, out string[] servers, out string[] froms, out string[] filenames, out uint64[] sizes, out uint64[] progresses, out uint64[] speeds, out uint64[] statuses) throws GLib.IOError;

    [DBus (name = "dcc_send_accept")]
    public abstract void dcc_send_accept (uint64 id) throws GLib.IOError;

    [DBus (name = "dcc_send_get")]
    public abstract string dcc_send_get (uint64 id, string key) throws GLib.IOError;

    [DBus (name = "dcc_send_remove")]
    public abstract void dcc_send_remove (uint64 id) throws GLib.IOError;

    [DBus (name = "dcc_send_resume")]
    public abstract void dcc_send_resume (uint64 id) throws GLib.IOError;

    [DBus (name = "dcc_send_set")]
    public abstract void dcc_send_set (uint64 id, string key, string value) throws GLib.IOError;

    [DBus (name = "ignore")]
    public abstract void ignore (string server, string pattern) throws GLib.IOError;

    [DBus (name = "ignores")]
    public abstract string[] ignores (string server) throws GLib.IOError;

    [DBus (name = "invite")]
    public abstract void invite (string server, string channel, string who) throws GLib.IOError;

    [DBus (name = "join")]
    public abstract void join (string server, string channel, string key) throws GLib.IOError;

    [DBus (name = "kick")]
    public abstract void kick (string server, string channel, string who, string message) throws GLib.IOError;

    [DBus (name = "list")]
    public abstract void list (string server, string channel) throws GLib.IOError;

    [DBus (name = "log")]
    public abstract string[] log (string server, string target, uint64 lines) throws GLib.IOError;

    [DBus (name = "message")]
    public abstract void message (string server, string target, string message) throws GLib.IOError;

    [DBus (name = "mode")]
    public abstract void mode (string server, string target, string mode) throws GLib.IOError;

    [DBus (name = "names")]
    public abstract void names (string server, string channel) throws GLib.IOError;

    [DBus (name = "nick")]
    public abstract void nick (string server, string nick) throws GLib.IOError;

    [DBus (name = "nickserv")]
    public abstract void nickserv (string server) throws GLib.IOError;

    [DBus (name = "notice")]
    public abstract void notice (string server, string target, string message) throws GLib.IOError;

    [DBus (name = "oper")]
    public abstract void oper (string server, string name, string password) throws GLib.IOError;

    [DBus (name = "part")]
    public abstract void part (string server, string channel, string message) throws GLib.IOError;

    [DBus (name = "quit")]
    public abstract void quit (string server, string message) throws GLib.IOError;

    [DBus (name = "raw")]
    public abstract void raw (string server, string command) throws GLib.IOError;

    [DBus (name = "server_get")]
    public abstract string server_get (string server, string group, string key) throws GLib.IOError;

    [DBus (name = "server_get_list")]
    public abstract string[] server_get_list (string server, string group, string key) throws GLib.IOError;

    // server_list ("", "") gives a list of servers known to maki
    [DBus (name = "server_list")]
    public abstract string[] server_list (string server, string group) throws GLib.IOError;

    [DBus (name = "server_remove")]
    public abstract void server_remove (string server, string group, string key) throws GLib.IOError;

    [DBus (name = "server_rename")]
    public abstract void server_rename (string old, string new_) throws GLib.IOError;

    [DBus (name = "server_set")]
    public abstract void server_set (string server, string group, string key, string value) throws GLib.IOError;

    [DBus (name = "server_set_list")]
    public abstract void server_set_list (string server, string group, string key, string[] list) throws GLib.IOError;

    [DBus (name = "servers")]
    public abstract string[] servers () throws GLib.IOError;

    [DBus (name = "shutdown")]
    public abstract void shutdown (string message) throws GLib.IOError;

    [DBus (name = "support_chantypes")]
    public abstract string support_chantypes (string server) throws GLib.IOError;

    [DBus (name = "support_prefix")]
    public abstract string[] support_prefix (string server) throws GLib.IOError;

    [DBus (name = "topic")]
    public abstract void topic (string server, string channel, string topic) throws GLib.IOError;

    [DBus (name = "unignore")]
    public abstract void unignore (string server, string pattern) throws GLib.IOError;

    [DBus (name = "user_away")]
    public abstract bool user_away (string server, string nick) throws GLib.IOError;

    [DBus (name = "user_channel_mode")]
    public abstract string user_channel_mode (string server, string channel, string nick) throws GLib.IOError;

    [DBus (name = "user_channel_prefix")]
    public abstract string user_channel_prefix (string server, string channel, string nick) throws GLib.IOError;

    [DBus (name = "user_from")]
    public abstract string user_from (string server, string nick) throws GLib.IOError;

    [DBus (name = "version")]
    public abstract uint64[] version () throws GLib.IOError;

    [DBus (name = "who")]
    public abstract void who (string server, string mask, bool operators_only) throws GLib.IOError;

    [DBus (name = "whois")]
    public abstract void whois (string server, string mask) throws GLib.IOError;

    [DBus (name = "action")]
    public signal void event_action (int64 time, string server, string from, string target, string message);

    [DBus (name = "away")]
    public signal void event_away (int64 time, string server);

    [DBus (name = "away_message")]
    public signal void event_away_message (int64 time, string server, string nick, string message);

    [DBus (name = "back")]
    public signal void event_back (int64 time, string server);

    [DBus (name = "banlist")]
    public signal void event_banlist (int64 time, string server, string channel, string mask, string who, int64 when);

    [DBus (name = "cannot_join")]
    public signal void event_cannot_join (int64 time, string server, string channel, string reason);

    [DBus (name = "connect")]
    public signal void event_connect (int64 time, string server);

    [DBus (name = "connected")]
    public signal void event_connected (int64 time, string server);

    [DBus (name = "ctcp")]
    public signal void event_ctcp (int64 time, string server, string from, string target, string message);

    [DBus (name = "dcc_send")]
    public signal void event_dcc_send (int64 time, string server, uint64 id, string from, string filename, uint64 size, uint64 progress, uint64 speed, uint64 status);

    [DBus (name = "error")]
    public signal void event_error (int64 time, string server, string domain, string reason, string[] arguments);

    [DBus (name = "invite")]
    public signal void event_invite (int64 time, string server, string from, string channel, string who);

    [DBus (name = "join")]
    public signal void event_join (int64 time, string server, string from, string channel);

    [DBus (name = "kick")]
    public signal void event_kick (int64 time, string server, string from, string channel, string who, string message);

    [DBus (name = "list")]
    public signal void event_list (int64 time, string server, string channel, int64 users, string topic);

    [DBus (name = "message")]
    public signal void event_message (int64 time, string server, string from, string target, string message);

    [DBus (name = "mode")]
    public signal void event_mode (int64 time, string server, string from, string target, string mode, string parameter);

    [DBus (name = "motd")]
    public signal void event_motd (int64 time, string server, string message);

    [DBus (name = "names")]
    public signal void event_names (int64 time, string server, string channel, string[] nicks, string[] prefixes);

    [DBus (name = "nick")]
    public signal void event_nick (int64 time, string server, string from, string new_nick);

    [DBus (name = "no_such")]
    public signal void event_no_such (int64 time, string server, string target, string type);

    [DBus (name = "notice")]
    public signal void event_notice (int64 time, string server, string from, string target, string message);

    [DBus (name = "oper")]
    public signal void event_oper (int64 time, string server);

    [DBus (name = "part")]
    public signal void event_part (int64 time, string server, string from, string channel, string message);

    [DBus (name = "quit")]
    public signal void event_quit (int64 time, string server, string from, string message);

    [DBus (name = "shutdown")]
    public signal void event_shutdown (int64 time);

    [DBus (name = "topic")]
    public signal void event_topic (int64 time, string server, string from, string channel, string topic);

    [DBus (name = "user_away")]
    public signal void event_user_away (int64 time, string server, string from, bool away);

    [DBus (name = "whois")]
    public signal void event_whois (int64 time, string server, string nick, string message);
}
