/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

/**
 * Static class for managing the launcher using libunity.
 */
public class Cable.Services.Launcher {

    static Unity.LauncherEntry launcher_entry;

    /**
     * Number shown in the badge on the launcher.
     * If the value is zero, the badge will be hidden automatically.
     */
    public static int64 jobs {
        get { return launcher_entry.count; }
        set {
            assert (value >= 0);
            launcher_entry.count = value;
            launcher_entry.count_visible = value > 0;
        }
    }

    /**
     * Progress shown on the launcher. Will be hidden when complete.
     */
    public static double progress {
        get { return launcher_entry.progress; }
        set {
            assert (0.0 <= value <= 1.0);
            launcher_entry.progress = value;
            launcher_entry.progress_visible = value < 1.0;
        }
    }
    
    /**
     * Set launcher to urgent.
     */
    public static bool urgent {
        get { return launcher_entry.urgent; }
        set { launcher_entry.urgent = value; }
    }

    /**
     * Must be called before anything else.
     */
    public static void init () {
        launcher_entry = Unity.LauncherEntry.get_for_desktop_id ("cable.desktop");
    }
}
