namespace Cable.Utils.ColorCodeParser {

    const string TOKEN = "&amp;#x3;";

    const string[] palette = {
        "#EEE",
        "#000",
        "#00007F",
        "#009300",
        "#FF0000",
        "#7F0000",
        "#9C009C",
        "#FC7F00",
        "#FFFF00",
        "#00FC00",
        "#009393",
        "#00FFFF",
        "#0000FC",
        "#FF00FF",
        "#D2D2D2"
    };

    string text;
    StringBuilder builder;
    int offset;
    bool should_close_tag;

    public string parse_text (string _text) {
        init (_text);
        
        while (true) {
            append_until_next_token_or_end ();
            close_tag_if_necessary ();

            if (end_reached ())
                break;

            open_tag ();
        }

        return builder.str;
    }
    
    void init (string _text) {
        text = _text;
        builder = new StringBuilder ();
        offset = 0;
        should_close_tag = false;
    }
    
    void append_until_next_token_or_end () {
        var next_token_index = get_next_token_index ();

        if (next_token_index != offset)
            builder.append (text[offset:next_token_index]);

        offset = next_token_index + TOKEN.length;
    }

    int get_next_token_index () {
        var result = text.index_of (TOKEN, offset);
        if (result == -1)
            result = text.length;
        return result;
    }

    void close_tag_if_necessary () {
        if (should_close_tag) {
            close_tag ();
            should_close_tag = false;
        }
    }

    void close_tag () {
        builder.append("</span>");
    }

    bool end_reached () {
        return offset >= text.length;
    }

    bool end_not_reached () {
        return ! end_reached ();
    }

    void open_tag () {
        string? fg = get_foreground_color ();
        string? bg = get_background_color ();

        should_close_tag = (fg != null) || (bg != null);

        if ( fg != null && bg != null ) {
            builder.append("<span color=\"%s\" background=\"%s\">".printf(fg,bg));
        } else if ( fg != null ) {
            builder.append("<span color=\"%s\">".printf(fg));
        } else if ( bg != null )  {
            builder.append("<span background=\"%s\">".printf(bg));
        }
    }

    string? get_foreground_color () {
        var color_code = -1;
        color_code = get_following_number ();

        if (is_valid_color_code (color_code))
            return palette[color_code];

        return (string) null;
    }

    string? get_background_color () {
        var color_code = -1;

        if ((offset+1) < text.length && text[offset] == ',') {
            offset++;

            color_code = get_following_number ();
        }

        if (is_valid_color_code (color_code))
            return palette[color_code];

        return (string) null;
    }

    int get_following_number () {
        var builder = new StringBuilder ();

        while (text[offset].isdigit() && end_not_reached ()) {
            builder.append_c (text[offset]);
            offset++;
        }

        if (builder.str.length > 0)
            return int.parse (builder.str);

        return -1;
    }

    bool is_valid_color_code (int color_code) {
        return color_code < palette.length && color_code >= 0;
    }
}
