/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Utils.MessageHistory {

    Gee.List <string> data;
    int current;
    
    public MessageHistory () {
        data = new Gee.ArrayList <string> ();
        current = 0;
    }

    // data[data.size] is an imaginary, empty string, which is
    // always the most recent message
    public void append (string message) {
        data.add (message);
        current = data.size;
    }
    
    public string get_current ()
        requires (0 <= current <= data.size) {
        if (current == data.size)
            return "";
        return data[current];
    }
    
    public string get_older ()
        requires (has_older ()) {
        set_current_to_older ();
        return get_current ();
    }
    
    public string get_newer ()
        requires (has_newer ()) {
        set_current_to_newer ();
        return get_current ();
    }
    
    public bool has_older () {
        return current > 0;
    }
    
    public bool has_newer () {
        return current < data.size;
    }
    
    void set_current_to_older ()
        requires (has_older ()) {
        current--;
    }
    
    void set_current_to_newer ()
        requires (has_newer ()) {
        current++;
    }

    public void modify_current (string value)
        requires (0 <= current < data.size) {
        data[current] = value;
    }
}
