namespace Cable.Utils.UrlUpMarker {

    // Following strings are used to build RegEx for matching URIs
    const string USERCHARS = "-[:alnum:]";
    const string USERCHARS_CLASS = "[" + USERCHARS + "]";
    const string PASSCHARS_CLASS = "[-[:alnum:]\\Q,?;.:/!%$^*&~\"#'\\E]";
    const string HOSTCHARS_CLASS = "[-[:alnum:]]";
    const string HOST = HOSTCHARS_CLASS + "+(\\." + HOSTCHARS_CLASS + "+)*";
    const string PORT = "(?:\\:[[:digit:]]{1,5})?";
    const string PATHCHARS_CLASS = "[-[:alnum:]\\Q_$.+!*,;:@&=?/~#%\\E]";
    const string PATHTERM_CLASS = "[^\\Q]'.}>) \t\r\n,\"\\E]";
    const string SCHEME = """(?:news:|telnet:|nntp:|file:\/|https?:|ftps?:|sftp:|webcal:
                             |irc:|sftp:|ldaps?:|nfs:|smb:|rsync:|ssh:|rlogin:|telnet:|git:
                             |git\+ssh:|bzr:|bzr\+ssh:|svn:|svn\+ssh:|hg:|mailto:|magnet:)""";

    const string USERPASS = USERCHARS_CLASS + "+(?:" + PASSCHARS_CLASS + "+)?";
    const string URLPATH =  "(?:(/" + PATHCHARS_CLASS + "+(?:[(]" + PATHCHARS_CLASS + "*[)])*" + PATHCHARS_CLASS + "*)*" + PATHTERM_CLASS + ")?";

    static const string[] regex_strings = {
        SCHEME + "//(?:" + USERPASS + "\\@)?" + HOST + PORT + URLPATH,
        "(?:www|ftp)" + HOSTCHARS_CLASS + "*\\." + HOST + PORT + URLPATH,
        "(?:callto:|h323:|sip:)" + USERCHARS_CLASS + "[" + USERCHARS + ".]*(?:" + PORT + "/[a-z0-9]+)?\\@" + HOST,
        "(?:mailto:)?" + USERCHARS_CLASS + "[" + USERCHARS+ ".]*\\@" + HOSTCHARS_CLASS + "+\\." + HOST,
        "(?:news:|man:|info:)[[:alnum:]\\Q^_{|}~!\"#$%&'()*+,./;:=?`\\E]+"
    };

    public string markup_urls (string text) {
        string result = "";

        try {
            foreach (var regex_string in regex_strings) {
                var regex = new Regex (@"($regex_string)");
                if (!regex.match (text))
                    continue;
                result = regex.replace (text, -1, 0, "<a href=\"\\1\">\\1</a>");
                return result;
            }
        } catch (GLib.RegexError error) {
            warning (error.message);
        }

        return text;
    }
}