/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public interface Cable.View.Window : GLib.Object {

    public abstract bool welcome_shown { get; set; }

    /**
     * List of all channels of this server.
     */
    public abstract Gee.ArrayList <Server> servers { owned get; }

    /**
     * Names of all owned channels.
     */
    public string[] server_ids {
        owned get {
            string[] result = {};
            foreach (var server in servers)
                result += server.id;
            return result;
        }
    }

    /**
     * Get the server of the currently selected channel.
     */
    public abstract Server? current_server { get; }

    /**
     * Get the currently selected channel.
     */
    public abstract Chat? current_chat { get; set; }

    /**
     * Amount of opened servers.
     */
    public uint n_servers {
        get { return servers.size; }
    }

    /**
     * Window title.
     */
    public abstract string title { get; set; }


    public signal void remove_channel ();

    public signal void add_channel ();

    public signal void selected (Chat chat);

    public signal void show_preferences ();

    /**
     * Join channel with given address and nick.
     * @param address Address of the network to be joined.
     * @param nick Nick to be used when joining.
     * @param channel_name Name of the channel to be joined.
     */
    public signal void join (string address, string nick, string channel_name);

    /**
     * Window has been closed.
     */
    public signal void close ();

    /**
     * Create and add a new channel to this server.
     * @param channel_name Name of the new channel.
     */
    public abstract Server? add_server (string server_name, string server_id);

    /**
     * Remove a channel from this server.
     * @param channel_name Name of the channel to be removed.
     */
    public abstract void remove_server (string server_id);

    /**
     * Check whether this server has a given server.
     * @param channel_name Name of the channel you are looking for.
     * @return True if the channel exists, else false.
     */
    public bool has_server (string server_id) {
        return server_id in server_ids;
    }

    /**
     * Retrieve a channel by its name.
     *
     * @param channel_name Name of the channel you are looking for.
     * @return The channel or null if not found.
     */
    public Server? get_server_by_id (string server_id) {
        foreach (var server in servers)
            if (server.id == server_id)
                return server;

        return (Server) null;
    }
}
