/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Widgets.Chat : Granite.Widgets.SourceList.Item {

    public Room room;

    public string nick { get; set; }
    public string id { get; set; }
    public string entry { get; set; }

    public int unread_messages {
        get { return int.parse (badge); }
        set { badge = (value <= 0) ? "" : value.to_string (); }
    }

    public bool has_focus {
        get { return room.text_entry.has_focus; }
    }

    public void rename_user (string? old_name, string new_name, UserType type, bool silent = false) {
        room.rename_user (old_name, new_name, type, silent);
    }

    public void set_away_status (string nick, bool away) {
        room.chat.message_away (nick, away);
    }

    public void message (owned string from, string message, MessageType type = MessageType.NORMAL) {
        room.chat.add_message (from, message, type);
    }
}

public class Cable.Widgets.PrivateChat : Widgets.Chat, View.Chat, View.PrivateChat {
    public PrivateChat (string name, string id) {
        GLib.Object (name: name, id: id);
        this.room = new Room (name);
        
        room.send_message.connect ((msg) => send_message (msg));
        room.text_entry.grab_focus.connect (() => { focused (); });
        
        // bind properties
        bind_property ("nick", room, "nick", GLib.BindingFlags.BIDIRECTIONAL);
        bind_property ("name", room, "name", GLib.BindingFlags.BIDIRECTIONAL);
        bind_property ("entry", room, "entry", GLib.BindingFlags.BIDIRECTIONAL);
    }
}

public class Cable.Widgets.Channel : Widgets.Chat, View.Chat, View.Channel {

    Gtk.Menu menu;
    Gtk.MenuItem leave_menu_item;

    public State state { get; set; }
    public string topic { get; set; }

    public Channel (string name, string id) {
        GLib.Object (name: name, id: id);
        this.room = new Room (name);

        this.menu = new Gtk.Menu ();
        this.leave_menu_item = new Gtk.MenuItem.with_label (_("Leave Channel"));
        menu.append (leave_menu_item);
        menu.show_all ();

        // setup signals
        leave_menu_item.activate.connect (()
            => part ());
            
        room.send_message.connect ((msg)
            => send_message (msg));
            
        room.rejoin.connect (()
            => join ());
            
        room.text_entry.grab_focus.connect (()
            => { focused (); });
        
        room.user_list.private_chat_requested.connect ((name)
            => private_chat_requested (name));

        // bind properties
        bind_property ("state", room, "state", GLib.BindingFlags.BIDIRECTIONAL);
        bind_property ("nick", room, "nick", GLib.BindingFlags.BIDIRECTIONAL);
        bind_property ("name", room, "name", GLib.BindingFlags.BIDIRECTIONAL);
        bind_property ("topic", room.topic_bar, "topic", GLib.BindingFlags.BIDIRECTIONAL);
        bind_property ("entry", room, "entry", GLib.BindingFlags.BIDIRECTIONAL);
    }

    public override Gtk.Menu? get_context_menu () {
        if (menu != null && menu.get_attach_widget () != null)
            menu.detach ();
        return menu;
    }

    public bool has_user (string nick) {
        return room.has_user (nick);
    }

    public void list_users (string[] nicks, string[] prefixes) {
        room.list_users (nicks, prefixes);
    }

    public void add_user (string name, UserType type, bool silent = false) {
        room.add_user (name, type, silent);
    }

    public void remove_user (string name, string message, bool silent = false) {
        room.remove_user (name, message, silent);
    }

}
