/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public enum Cable.MessageType {
    NORMAL,
    ENTER,
    LEAVE,
    SUGGESTION,
    NICK,
    AWAY,
    PING,
    ACTION;
    /* QUIT, KICK, BAN, MODE(?), ACTION
    */
    
    public string color () {
        switch (this) {
            case NORMAL:
            case ACTION:
                return "#333";
            case ENTER:
                return "#9a9";
            case LEAVE:
                return "#a99";
            case SUGGESTION:
            case NICK:
            case AWAY:
                return "#999";
            case PING:
                return "#da4d45";
        }
        
        return "";
    }
}

public class Cable.Widgets.ChatDisplay : Gtk.EventBox {

    private Gtk.Grid main_grid;
    private int line = 0;

    public ChatDisplay () {
        this.get_style_context ().add_class (Granite.StyleClass.CONTENT_VIEW);
        main_grid = new Gtk.Grid ();
        main_grid.margin = 12;
        main_grid.margin_bottom = 0;
        main_grid.column_spacing = 12;
        main_grid.row_spacing = 6;
        main_grid.expand = true;
        expand = true;
        this.add (main_grid);
    }

    public void add_message (string _user, string _message, MessageType type = MessageType.NORMAL) {
        var message = Utils.parse_string_to_markup (_message);
        var message_color = type.color ();
        var nick_color = get_color_for_nick (_user);
        var time = new DateTime.now_local ();
        var stamp = time.format ("%T");
        
        message = @"<span color='$message_color'>$message</span>";
        var user = @"<span color='$nick_color'>$_user</span>";
    
        var user_label = new Gtk.Label ("");
        user_label.set_markup (user);
        user_label.set_alignment (1, 0);
        user_label.width_request = 80;
        user_label.ellipsize = Pango.EllipsizeMode.END;
        
        var message_label = new Granite.Widgets.WrapLabel ("");
        message_label.hexpand = true;
        message_label.use_markup = true;
        message_label.selectable = true;
        message_label.set_markup (message);
        message_label.wrap_mode = Pango.WrapMode.WORD;
        message_label.wrap = true;
        
        var time_label = new Gtk.Label ("");
        time_label.set_markup (@"<span color='#bbb'>$stamp</span>");
        time_label.set_alignment (1, 0);
        
        main_grid.attach (user_label, 0, line, 1, 1);
        main_grid.attach (message_label, 1, line, 1, 1);
        main_grid.attach (time_label, 2, line, 1, 1);
        
        this.show_all ();
        
        line++;
    }
    
    public void message_part (string nick, string message) {
        add_message ("", @"← $nick: $message", MessageType.LEAVE);
    }
    
    public void message_join (string nick) {
        add_message ("", @"→ $nick", MessageType.ENTER);
    }
    
    public void message_nick (string old_nick, string new_nick) {
        add_message ("", @"$old_nick → $new_nick", MessageType.NICK);
    }
    
    public void message_suggestions (string[] suggestions) {
        add_message ("", string.joinv (" - ", suggestions), MessageType.SUGGESTION);
    }
    
    public void message_away (string nick, bool away) {
        add_message ("", nick + " " + _("is") + " " + (away ? _("away") : _("back")), MessageType.AWAY);
    }
    
    Gee.HashMap <string, string> color_map = null;
    int i = 0;
    
    string get_color_for_nick (string nick) {
        if (color_map == null)
            color_map = new Gee.HashMap <string, string> ();
        
        if (nick == "")
            return "#000";
        
        string[] colors = {
            "#990055",
            "#669900",
            "#ec6f00",
            "#0077aa",
            "#dd1144",
            "#00a0b3",
            "#8f0a0a"
        };
        
        if (!color_map.has_key (nick)) {
            color_map[nick] = colors[i++ % colors.length];
        }
        
        return color_map[nick];
    }
}