/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

/**
 * This class represents a connection to a network. It is an expandable item in the
 * source list.
 */
public class Cable.Widgets.Server : Granite.Widgets.SourceList.ExpandableItem, View.Server {

    public Gee.ArrayList <View.Chat> chats {
        owned get {
            var _chats = new Gee.ArrayList <View.Chat> ();
            foreach (var child in children)
                _chats.add (child as View.Chat);
            return _chats;
       }
    }

    public bool away {
        set {
            away_menu_item.label = (value) ? _("Mark Present") : _("Mark Away");
            away_menu_item.show_all ();
        }
    }
    
    public string id { get; set; }

    Gtk.Menu menu;
    Gtk.MenuItem away_menu_item;
    Gtk.MenuItem part_menu_item;

    internal Server (string name, string id) {
        GLib.Object (name: name, id: id);
        setup_contextmenu ();
    }

    void setup_contextmenu ()
        requires (menu == null)
        ensures (menu != null) {
        
        this.menu = new Gtk.Menu ();

        this.away_menu_item = new Gtk.MenuItem.with_label (_("Mark Away"));
        this.part_menu_item = new Gtk.MenuItem.with_label (_("Leave All Channels"));

        menu.append (away_menu_item);
        menu.append (part_menu_item);

        away_menu_item.activate.connect (() => toggle_away ());
        part_menu_item.activate.connect (() => part_all ());

        menu.show_all ();
    }

    public override Gtk.Menu? get_context_menu () {
        if (menu != null && menu.get_attach_widget () != null)
            menu.detach ();
        return menu;
    }
    
    public void add_channel (string chat_name, string chat_id)
        requires (!has_chat (chat_name))
        ensures (has_chat (chat_name)) {

        var chat = new Channel (chat_name, chat_id);
        add (chat);
    }
    
    public void add_private_chat (string chat_name, string chat_id)
        requires (!has_chat (chat_name))
        ensures (has_chat (chat_name)) {

        var chat = new PrivateChat (chat_name, chat_id);
        add (chat);
    }

    public void remove_chat (string chat_name)
        requires (has_chat (chat_name))
        ensures (!has_chat (chat_name)) {

        var chat = get_chat (chat_name) as Widgets.Chat;
        remove (chat);
    }
}
