/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Widgets.UserList : Granite.Widgets.SourceList {

    Granite.Widgets.SourceList.ExpandableItem operators;
    Granite.Widgets.SourceList.ExpandableItem voiced;
    Granite.Widgets.SourceList.ExpandableItem regular;

    delegate void ForeachFunction (User user);

    public Gee.Set <string> nicks {
        owned get {
            var _nicks = new Gee.HashSet <string> ();
            @foreach ((child) => _nicks.add (child.name));
            return _nicks as Gee.Set;
        }
    }
    
    public signal void private_chat_requested (string user_name);

    public UserList () {
        operators = new Granite.Widgets.SourceList.ExpandableItem (_("Operators"));
        voiced = new Granite.Widgets.SourceList.ExpandableItem (_("Voiced"));
        regular = new Granite.Widgets.SourceList.ExpandableItem (_("Regular"));

        regular.expanded = true;
        operators.expanded = true;
        voiced.expanded = true;

        hscrollbar_policy = Gtk.PolicyType.NEVER;
        width_request = 150;
        set_sort_func ((a,b) => {
            return a.name.collate (b.name);
        });

        root.add (operators);
        root.add (voiced);
        root.add (regular);
        
        no_show_all = true;
        visible = false;
    }
    
    public bool has_user (string name) {
        return name in nicks;
    }

    public void add_user (string name, UserType user_type = UserType.REGULAR) {
        var user = new User (name);
        user.selectable = false;

        switch (user_type) {
            case UserType.OPERATOR:
                operators.add (user);
                break;
            case UserType.VOICED:
                voiced.add (user);
                break;
            default:
                regular.add (user);
                break;
        }
        
        user.private_chat_requested.connect (() => {
            private_chat_requested (user.name);
        });
        
        update_user_counts ();
        
        if (visible == false) {
            no_show_all = false;
            visible = true;
        }
    }
    
    public void remove_user (string name) {
        @foreach ((child) => {
            if (child.name != name)
                return;
            child.parent.remove (child);
            update_user_counts ();
        });
        
        if (nicks.size == 0) {
            no_show_all = true;
            visible = false;
        }
    }
    
    void update_user_counts () {
        regular.name = _("Regular") + @" ($(regular.n_children.to_string ()))";
        operators.name = _("Operators") + @" ($(operators.n_children.to_string ()))";
        voiced.name = _("Voiced") + @" ($(voiced.n_children.to_string ()))";
    }
    
    new void @foreach (ForeachFunction func) {
        Granite.Widgets.SourceList.ExpandableItem[] lists = {
            voiced, operators, regular
        };
        
        foreach (var list in lists)
            foreach (var child in list.children)
                func (child as User);
    }
}
