/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Widgets.Welcome : Granite.Widgets.Welcome {

    public signal void done (string address, string nick, string channel);

    Gtk.Entry channel_entry;
    Gtk.Button connect_button;
    Gtk.ComboBoxText server_entry;
    Gtk.ComboBoxText id_entry;

    public Welcome () {
        base (_("Connect to IRC"), _("Select a channel to join and begin to talk."));

        var grid = new Gtk.Grid ();
        grid.margin = 12;
        grid.column_spacing = 6;
        grid.row_spacing = 6;

        // Labels
        var server_label = new Gtk.Label (_("Network:"));
        server_label.halign = Gtk.Align.END;
        server_label.valign = Gtk.Align.CENTER;

        var id_label = new Gtk.Label (_("Identity:"));
        id_label.halign = Gtk.Align.END;
        id_label.valign = Gtk.Align.CENTER;

        var channel_label = new Gtk.Label (_("Channel:"));
        channel_label.halign = Gtk.Align.END;
        channel_label.valign = Gtk.Align.CENTER;

        connect_button = new Gtk.Button.with_label (_("Connect"));
        connect_button.get_style_context ().add_class ("suggested-action");
        connect_button.sensitive = false;

        var connect_button_grid = new Gtk.Grid ();
        var connect_fake_label = new Gtk.Label ("");
        connect_fake_label.hexpand = true;
        connect_button_grid.attach (connect_fake_label, 0, 0, 1, 1);
        connect_button_grid.attach (connect_button, 1, 0, 1, 1);

        // Entries/Comboboxes
        server_entry = new Gtk.ComboBoxText.with_entry ();
        for (int i = 0; i < Utils.Network.all.size; i++)
            server_entry.append (i.to_string (), Utils.Network.all[i].name);
        server_entry.active = 0;

        id_entry = new Gtk.ComboBoxText.with_entry ();
        for (int i = 0; i < Utils.Identity.all.size; i++) {
            id_entry.append (i.to_string (), Utils.Identity.all[i].nick_name);
            if (Utils.Identity.all[i].nick_name == Settings.preferences.last_used_nick)
                id_entry.active = i;
        }
        
        if (Settings.preferences.last_used_nick == "")
            id_entry.active = 0;

        channel_entry = new Gtk.Entry ();
        channel_entry.text = "#";
        channel_entry.width_request = 250;

        grid.attach (server_label, 0, 0, 1, 1);
        grid.attach (server_entry, 1, 0, 1, 1);

        grid.attach (id_label, 0, 1, 1, 1);
        grid.attach (id_entry, 1, 1, 1, 1);
        grid.attach (channel_label, 0, 2, 1, 1);
        grid.attach (channel_entry, 1, 2, 1, 1);
        grid.attach (connect_button_grid, 1, 3, 1, 1);
        grid.margin = 6;

        channel_entry.insert_text.connect ((text) => {
            Utils.filter (channel_entry, text, {" ", ":"});
        });
        id_entry.changed.connect (() => {
            if (channel_entry.text.length > 1 && server_entry.get_active_text ().length > 0 && id_entry.get_active_text ().length > 0) {
                connect_button.sensitive = true;
            } else {
                connect_button.set_sensitive (false);
            };
        });
        server_entry.changed.connect (() => {
            if (channel_entry.text.length > 1 && server_entry.get_active_text ().length > 0 && id_entry.get_active_text ().length > 0) {
                connect_button.sensitive = true;
            } else {
                connect_button.set_sensitive (false);
            };
        });
        channel_entry.changed.connect (() => {
            if (channel_entry.text.length > 1 && server_entry.get_active_text ().length > 0 && id_entry.get_active_text ().length > 0) {
                connect_button.sensitive = true;
            } else {
                connect_button.set_sensitive (false);
            };
        });

        connect_button.clicked.connect (() => channel_entry.activate ());
        channel_entry.activate.connect (() => {
            Utils.Network network;
            Utils.Identity identity;
            
            if (server_entry.active_id == null) {
                network = new Utils.Network (server_entry.get_active_text ());
            } else {
                network = Utils.Network.all[int.parse (server_entry.active_id)];
            }

            if (id_entry.active_id == null) {
                identity = new Utils.Identity (id_entry.get_active_text ());
            } else {
                identity = Utils.Identity.all[int.parse (id_entry.active_id)];
            }
            
            done (network.address, identity.nick_name, channel_entry.text);
        });

        options.add (grid);
        this.show_all ();
    }

    public void focus_channel_entry () {
        channel_entry.grab_focus ();
        channel_entry.set_position (-1);
    }
}
