class TestMessageHistory : Unit.TestCase {

    Cable.Utils.MessageHistory history;

    public TestMessageHistory () {
        base ("Message History");

        add_test_method ("Initial State", test_initial_state);
        add_test_method ("First Insert", test_first_insert);
        add_test_method ("Second Insert", test_second_insert);
        add_test_method ("Navigate Back", test_navigate_back);
        add_test_method ("Navigate Forth", test_navigate_forth);
        add_test_method ("Reset After Insert", test_reset_after_insert);
        add_test_method ("Modify Current", test_modify_current);
    }

    public override void set_up () {
        history = new Cable.Utils.MessageHistory ();
    }

    public void test_initial_state () throws Assert.Error {
        Assert.is_false (history.has_older ());
        Assert.is_false (history.has_newer ());
        Assert.equals ("", history.get_current ());
    }

    public void test_first_insert () throws Assert.Error {
        history.append ("one");

        Assert.equals ("", history.get_current ());
        Assert.is_false (history.has_newer ());
        Assert.is_true (history.has_older ());
    }

    public void test_second_insert () throws Assert.Error {
        history.append ("one");
        history.append ("two");

        Assert.equals ("", history.get_current ());
        Assert.is_false (history.has_newer ());
        Assert.is_true (history.has_older ());
    }

    public void test_navigate_back () throws Assert.Error {
        history.append ("one");

        Assert.equals ("one", history.get_older ());
        Assert.equals ("one", history.get_current ());
        Assert.is_true (history.has_newer ());
    }

    public void test_navigate_forth () throws Assert.Error {
        history.append ("one");
        history.get_older ();

        Assert.equals ("one", history.get_current ());
        Assert.equals ("", history.get_newer ());
        Assert.is_true (history.has_older ());
        Assert.is_false (history.has_newer ());
    }

    public void test_reset_after_insert () throws Assert.Error {
        history.append ("one");
        history.get_older ();
        history.append ("two");

        Assert.equals ("", history.get_current ());
        Assert.is_false (history.has_newer ());
        Assert.is_true (history.has_older ());
    }

    public void test_modify_current () throws Assert.Error {
        history.append ("one");
        history.get_older ();

        Assert.equals ("one", history.get_current ());
        Assert.is_true (history.has_newer ());
        Assert.is_false (history.has_older ());

        history.modify_current ("modified");

        Assert.equals ("modified", history.get_current ());
        Assert.is_true (history.has_newer ());
        Assert.is_false (history.has_older ());
    }
}
