using Cable.Utils;

class TestUpMarker : Unit.TestCase {

    UpMarker up_marker;

    public TestUpMarker () {
        base("UpMarker");

        add_test_method ("Color Codes - Foreground", test_foreground_color);
        add_test_method ("Color Codes - Foreground and Background", test_fg_and_bg_color);
        add_test_method ("Color Codes - Stop Colors", test_stop_colors);
        add_test_method ("Color Codes - Switch Colors", test_switch_colors);
        add_test_method ("URLs - Valid URLs", test_valid_urls);
        add_test_method ("URLs - Invalid URLs", test_invalid_urls);
    }

    public override void set_up () {
        up_marker = new UpMarker ();
    }

    string create_token (int foreground_color = -1, int background_color = -1) {
        var token = "&#x3;";

        if (foreground_color >= 0) {
            token += foreground_color.to_string ();
            
            if (background_color >= 0)
                token += "," + background_color.to_string ();
        }
        
        return token;
    }

    public void test_foreground_color () throws Assert.Error {
        var text = create_token (2) + "hello world!";
        var parsed_text = up_marker.mark_up (text);
        Assert.equals ("<span color=\"#00007F\">hello world!</span>",
                       parsed_text);
    }
    
    public void test_fg_and_bg_color () throws Assert.Error {
        var text = create_token (2, 3) + "hello world!";
        var parsed_text = up_marker.mark_up (text);
        Assert.equals ("<span color=\"#00007F\" background=\"#009300\">hello world!</span>",
                       parsed_text);
    }
    
    public void test_stop_colors () throws Assert.Error {
        var text = create_token (2, 3) + "hello" + create_token () + " world!";
        var parsed_text = up_marker.mark_up (text);
        Assert.equals ("<span color=\"#00007F\" background=\"#009300\">hello</span> world!",
                       parsed_text);
    }
    
    public void test_switch_colors () throws Assert.Error {
        var text = create_token (2, 3) + "hello" + create_token (6, 7) + " world!";
        var parsed_text = up_marker.mark_up (text);
        Assert.equals ("<span color=\"#00007F\" background=\"#009300\">hello</span>"
                     + "<span color=\"#9C009C\" background=\"#FC7F00\"> world!</span>",
                       parsed_text);
    }
    
    const string[] valid_urls = {
        "http://elementaryos.org/",
        "www.google.com",
        "mailto:test@elementaryos.com",
        "http://valadoc.org/#!api=gtk+-3.0/Gtk.main",
        "http://en.wikipedia.org/wiki/Ampersand#Text_markup",
        "https://www.google.com/search?client=ubuntu&channel=fs&q=vala&ie=utf-8&oe=utf-8"
    };

    public void test_valid_urls () throws Assert.Error {
        foreach (var url in valid_urls) {
            var parsed_text = up_marker.mark_up (url);
            var escaped_url = GLib.Markup.escape_text (url);
            Assert.equals (@"<a href=\"$escaped_url\">$escaped_url</a>", parsed_text);
        }
    }
    
    const string[] invalid_urls = {
        "test",
        "test@test",
        "test.com",
        "abc://test.test.com"
    };

    public void test_invalid_urls () throws Assert.Error {
        foreach (var url in invalid_urls) {
            var parsed_text = up_marker.mark_up (url);
            var escaped_url = GLib.Markup.escape_text (url);
            Assert.differs (@"<a href=\"$escaped_url\">$escaped_url</a>", parsed_text);
        }
    }
}
