namespace Assert {

    const string equals_format = """
  Equality Assertion Failed:
  > Should have been: <%s>
  > But was:          <%s>
  > %s
""";

    const string differs_format = """
  Difference Assertion Failed:
  > Should not have been: <%s>
  > %s
""";

    const string is_false_format = """
  Falsehood Assertion Failed:
  > %s
""";

    const string is_true_format = """
  Truth Assertion Failed:
  > %s
""";

    const string is_null_format = """
  Null Assertion Failed:
  > %s
""";

    const string is_not_null_format = """
  Not Null Assertion Failed:
  > %s
""";

    public errordomain Error {
        ASSERTION_FAILED
    }

    public void equals <T> (T value_1, T value_2, string _message = "")
    throws Error.ASSERTION_FAILED {
        if (typed_equals (value_1, value_2) == false) {
            var val_1 = typed_value_to_string (value_1);
            var val_2 = typed_value_to_string (value_2);
            var message = equals_format.printf (val_1, val_2, _message);
            fail (message);
        }
    }

    public void differs <T> (T value_1, T value_2, string _message = "")
    throws Error.ASSERTION_FAILED {
        if (typed_equals (value_1, value_2)) {
            var val_1 = typed_value_to_string (value_1);
            var message = differs_format.printf (val_1, _message);
            fail (message);
        }
    }

    public void is_false (bool value, string _message = "")
    throws Error.ASSERTION_FAILED {
        if (value) {
            var message = is_false_format.printf (_message);
            fail (message);
        }
    }

    public void is_true (bool value, string _message = "")
    throws Error.ASSERTION_FAILED {
        if (!value) {
            var message = is_true_format.printf (_message);
            throw new Error.ASSERTION_FAILED (message);
        }
    }

    public void is_null <T> (T value, string _message = "")
    throws Error.ASSERTION_FAILED {
        if (value != null) {
            var message = is_null_format.printf (_message);
            fail (message);
        }
    }

    public void is_not_null <T> (T value, string _message = "")
    throws Error.ASSERTION_FAILED {
        if (value == null) {
            var message = is_not_null_format.printf (_message);
            fail (message);
        }
    }

    string typed_value_to_string <T> (T value) {
        string result = "";
        Type value_type = typeof (T);

        if (value_type.is_value_type()) {
            if ( value_type.is_a( typeof(string) ) )
                result = (string) value;
            else if (value_type.is_a (typeof (int32)))
                result = ((int32) value).to_string ();
            else if (value_type.is_a (typeof(uint32)))
                result = ((uint32) value).to_string ();
            else if (value_type.is_a (typeof(int16)))
                result = ((int16) value).to_string ();
            else if (value_type.is_a (typeof(uint16)))
                result = ((uint16) value).to_string ();
            else if (value_type.is_a (typeof(int8)))
                result = ((int8) value).to_string ();
            else if (value_type.is_a (typeof(uint8)))
                result = ((uint8) value).to_string ();
            else if (value_type.is_a (typeof(bool)))
                result = ((bool) value).to_string ();
        }

        return result;
    }
    
    bool typed_equals <T> (T value_1, T value_2) {
        var type = typeof (T);
        var equal_func = get_equal_func_for (type);
        return equal_func (value_1, value_2);
    }

    GLib.EqualFunc get_equal_func_for (Type t) {
        if (t == typeof (string))
            return GLib.str_equal;
        else
            return GLib.direct_equal;
    }

    void fail (string message = "") throws Error.ASSERTION_FAILED {
        if (message == "")
            throw new Error.ASSERTION_FAILED ("Assertion Failed");
        else
            throw new Error.ASSERTION_FAILED (message);
    }
}
