public abstract class Unit.TestCase : GLib.Object, Test {

    Gee.ArrayList <UnitTest> tests;

    /**
     * Name of this TestCase for displaying.
     */
    public string name { get; set; }

    /**
     * Always 1 for TestCase.
     */
    public int test_cases { 
        get { return 1; }
    }
    
    public int n_tests {
        get { return tests.size; }
    }
    
    public TestCase (string name) {
        Object (name: name);
        tests = new Gee.ArrayList <UnitTest> ();
    }
    
    /**
     * Add a new test method and a name to display it.
     */
    public void add_test_method (string name, TestMethod method) {
        tests.add (new UnitTest (name, method));
    }
    
    /**
     * Run all test in this TestCase. No order guaranteed.
     */
    public void run () {
        print_start_test_case ();
        foreach (var unit_test in tests)
            run_unit_test (unit_test);
    }
    
    void run_unit_test (UnitTest unit_test) {
        start_unit_test (unit_test);
            
        try {
            unit_test.method ();
        } catch (Assert.Error e) {
            stop_unit_test_failed (e.message);
            return;
        }
        
        stop_unit_test_succeded ();
    }
    
    void start_unit_test (UnitTest unit_test) {
        print_start_unit_test (unit_test);
        set_up ();
    }
    
    void stop_unit_test_failed (string message) {
        print_failed_unit_test (message);
        tear_down ();
    }
    
    void stop_unit_test_succeded () {
        print_succeded_unit_test ();
        tear_down ();
    }
    
    void print_start_test_case () {
        print (format_status (@"- $name\n"));
    }
    
    void print_start_unit_test (UnitTest unit_test) {
        var test_name = unit_test.name;
        print (@"- $test_name ...");
    }
    
    void print_succeded_unit_test () {
        print (format_success (" OK ") + "\n");
    }
    
    void print_failed_unit_test (string message) {
        print (format_error (" [FAIL]" + message + "\n"));
    }
    
   /**
    * Sets up the fixture, for example, open a network connection.
    * This method is called before a test is executed.
    */
    protected virtual void set_up () {
    }
    

    /**
    * Tears down the fixture, for example, close a network connection.
    * This method is called after a test is executed.
    */
    protected virtual void tear_down () {
    }
    
    /**
     * Like set_up, but executed once before any tests have been run.
     */
    protected virtual void set_up_class () {
    }
    
    /**
     * Like tear_down, but executed once after all tests have finished.
     */
    protected virtual void tear_down_class () {
    }

    private class UnitTest : GLib.Object {
        public TestMethod method;
        public string name;
        
        public UnitTest (string name, TestMethod method) {
            this.name = name;
            this.method = method;
        }
    }
}
